
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;

import org.apache.log4j.Logger;
import org.drools.KnowledgeBase;
import org.drools.SystemEventListenerFactory;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.event.process.ProcessCompletedEvent;
import org.drools.event.process.ProcessEventListener;
import org.drools.event.process.ProcessNodeLeftEvent;
import org.drools.event.process.ProcessNodeTriggeredEvent;
import org.drools.event.process.ProcessStartedEvent;
import org.drools.event.process.ProcessVariableChangedEvent;
import org.drools.io.ResourceFactory;
import org.drools.logger.KnowledgeRuntimeLogger;
import org.drools.logger.KnowledgeRuntimeLoggerFactory;
import org.drools.runtime.StatefulKnowledgeSession;
import org.jbpm.process.workitem.wsht.SyncWSHumanTaskHandler;
import org.jbpm.task.TaskService;
import org.jbpm.task.User;
import org.jbpm.task.query.TaskSummary;
import org.jbpm.task.service.TaskServiceSession;
import org.jbpm.task.service.local.LocalTaskService;

public class LocalTaskClientTest {
	
	private static Logger logger = Logger.getLogger(LocalTaskClientTest.class);
	private static StatefulKnowledgeSession ksession;
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		try {
	        EntityManagerFactory emf = Persistence.createEntityManagerFactory("org.jbpm.task");
	        org.jbpm.task.service.TaskService taskService = new org.jbpm.task.service.TaskService(emf, SystemEventListenerFactory.getSystemEventListener());
	        TaskServiceSession taskSession = taskService.createSession();
//	        User user = new User();
//	        user.setId("frank");
//	        taskSession.addUser( user );
			// load up the knowledge base
			KnowledgeBase kbase = readKnowledgeBase();
			ksession = kbase.newStatefulKnowledgeSession();
			
			KnowledgeRuntimeLogger logger = KnowledgeRuntimeLoggerFactory.newThreadedFileLogger(ksession, "test", 1000);
			CustomProcessEventListener customProcessEventListener = new CustomProcessEventListener();
			ksession.addEventListener(customProcessEventListener);
			System.out.println("Registering Human Task..");
			
			SyncWSHumanTaskHandler syncWSHumanTaskHandler = new SyncWSHumanTaskHandler();
			TaskService localTaskService = new LocalTaskService(taskSession);
			syncWSHumanTaskHandler.setClient(localTaskService);
			ksession.getWorkItemManager().registerWorkItemHandler("Human Task", syncWSHumanTaskHandler);

			// start a new process instance
			Map<String, Object> params = new HashMap<String, Object>();
			params.put("username", "frank");
			params.put("actual_male", 1);
			params.put("actual_female", 3);
//			ksession.startProcess("humanTaskFlow", params);
			
			List<TaskSummary> taskSummaryList = localTaskService.getTasksAssignedAsPotentialOwner("frank", "en-UK");
			for (TaskSummary taskSummary : taskSummaryList) {
				System.out.println(taskSummary.getId() + "\t" + taskSummary.getName() + "\t" + taskSummary.getStatus());
				localTaskService.start(taskSummary.getId(), "frank");
				localTaskService.complete(taskSummary.getId(), "frank", null);	
			}

			logger.close();
			System.out.println((new Date()) + "\tProcess completed?");
//			System.exit(0);
		} catch (Throwable t) {
			t.printStackTrace();
		}
	}
	private static KnowledgeBase readKnowledgeBase() throws Exception {
		System.out.println("Loading knowledge base.");
		KnowledgeBuilder kbuilder = KnowledgeBuilderFactory.newKnowledgeBuilder();
		kbuilder.add(ResourceFactory.newClassPathResource("com/geneoz/procedureengine/HumanTaskFlow.bpmn"), ResourceType.BPMN2);
		return kbuilder.newKnowledgeBase();
	}
	
	private static class CustomProcessEventListener implements ProcessEventListener {

		@Override
		public void afterNodeLeft(ProcessNodeLeftEvent arg0) {
			// TODO Auto-generated method stub
			
		}

		@Override
		public void afterNodeTriggered(ProcessNodeTriggeredEvent arg0) {
			// TODO Auto-generated method stub
			
		}

		@Override
		public void afterProcessCompleted(ProcessCompletedEvent event) {
			// TODO Auto-generated method stub
			logger.debug("Process has completed.");
			ksession.dispose();
			System.exit(0);
		}

		@Override
		public void afterProcessStarted(ProcessStartedEvent arg0) {
			// TODO Auto-generated method stub
			
		}

		@Override
		public void afterVariableChanged(ProcessVariableChangedEvent arg0) {
			// TODO Auto-generated method stub
			
		}

		@Override
		public void beforeNodeLeft(ProcessNodeLeftEvent arg0) {
			// TODO Auto-generated method stub
			
		}

		@Override
		public void beforeNodeTriggered(ProcessNodeTriggeredEvent arg0) {
			// TODO Auto-generated method stub
			
		}

		@Override
		public void beforeProcessCompleted(ProcessCompletedEvent arg0) {
			// TODO Auto-generated method stub
			logger.debug("Process is going to be completed.");
		}

		@Override
		public void beforeProcessStarted(ProcessStartedEvent arg0) {
			// TODO Auto-generated method stub
			
		}

		@Override
		public void beforeVariableChanged(ProcessVariableChangedEvent arg0) {
			// TODO Auto-generated method stub
			
		}
		
	}
	
	private static class SystemEventListener implements org.drools.SystemEventListener {
		public void debug(String arg0) {
			logger.debug(arg0);
		}
		public void debug(String arg0, Object arg1) {
			logger.debug(arg0);
		}
		public void exception(Throwable arg0) {
			logger.error(arg0);
		}
		public void exception(String arg0, Throwable arg1) {
			logger.error(arg0, arg1);
		}
		public void info(String arg0) {
			logger.info(arg0);
		}
		public void info(String arg0, Object arg1) {
			logger.info(arg0);
		}
		public void warning(String arg0) {
			logger.warn(arg0);
		}
		public void warning(String arg0, Object arg1) {
			logger.warn(arg0);
		}
	}

}
